/* -*- mode:prolog; -*-
 ******************************************************************
 * $Id$
 * Copyright (C) 2008 by INRIA 
 * Author: Eric de la Clergerie <Eric.De_La_Clergerie@inria.fr>
 * ----------------------------------------------------------------
 *
 *  chart2.pl -- une autre implementation d'un algorithme a charte
 *
 * ----------------------------------------------------------------
 * Description
 * une implementation d'un algorithme a charte de type Earley
 * en s'appuyant sur un modle ask/tell grce au mta-prdicat '$answers'(G)
 *
 * ----------------------------------------------------------------
 */

%% Bibliothque pour les impressions
:-require 'format.pl'.

%% Bibliotheque pour les mta-call
%% :-require 'call.pl'.

:-extensional 'C'/3.

'C'([T|R],T,R).

%% Pour acceder directement aux productions de la grammaire
:-extensional (-->)/2.

%% Pour activer un mode verbose avec des message de trace
:-extensional verbose_opt.

%% Affiche un message si le mode verbose est activ
:-rec_prolog verbose/2.

verbose(Msg,Args) :-
   argv(L),
   ( verbose_opt ->
       format(Msg,Args)
   ;   
       true
   ).


%% earley_parse(Goal,L,R)
%% ----------
%% Le point de dpart dans l'algorithme pour l'analyse de Goal entre L et R
%%     - initialise l'agenda
%%     - process l'agenda jusqu'a ce qu'il soit vide
%%     - test la presence d'un item reponse pour Goal entre 
earley_parse(Goal,L,R) :-
	( prediction(Goal(L)),
	  fail
	;
	  process,
	  fail
	;
	  stored((Goal --> true)(L,R))
	).

%% prediction(A(I))
%% --------
%% Ajout au store des items rsultant d'une prdiction
%% pour un non terminal A  partir de la position I
%% en utilisant les production de la grammaire

%% process/0
%% Applique toute les rgles dductives
%%       Item1 Item2          Item1
%%       -----------   ou     -----
%%          Item3             Item3
%% en attendant la prsence de Item1 ou Item2
%% avant de pouvoir ajouter Item3 au store

:-rec_prolog scan/3.

scan([],L,L).
scan([Token|Tokens],L,R) :-
	'C'(L,Token,M),
	scan(Tokens,M,R).

%% stored(Item)
%% attend (mode ask) la prsence de Item dans la table

%% store(Item)
%% enregistre Item dans la table en mode tell

:-rec_prolog dcg_normalize/2.

dcg_normalize(Body,XBody) :-
	( Body = (B,Rest) ->
	    dcg_normalize(Rest,XRest),
	    XBody = (B,XRest)
	;
	    XBody = (Body,true)
	).

:-rec_prolog get_sentence/1.
get_sentence(Sentence) :-
	argv(L),
	( L=['-verbose'|Sentence] -> record(verbose_opt) ;  L = Sentence).

%% La requte,
%% ./chart2 french.dcg -- Paul mange une pomme
%% ./chart2 french.dcg -- -verbose Paul mange une pomme
?- get_sentence(Sentence),
    earley_parse(s,Sentence,[]).

:-rec_prolog lexicon/2.

:-public(lexicon/2).

%% lexicon(Form,Struct)
